@@ -339,7 +339,10 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
 
         # ssl variables
         self.ssl_certificate = None
+        # first ssl error
         self.ssl_errnum = 0
+        # all ssl errors
+        self.ssl_errors = []
 
         self.alpn = alpn
 
@@ -651,19 +654,21 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
             received = self.received_bytes_buff + received
             self.received_bytes_buff = b''
 
-        # try to decode data
-        try:
-            received = decode_py2(received, 'utf-8')
-        except UnicodeDecodeError:
-            for i in range(-1, -4, -1):
-                char = received[i]
-                if sys.version_info[0] < 3: # with py2 we get a str
-                    char = ord(char)
-                if char & 0xc0 == 0xc0:
-                    self.received_bytes_buff = received[i:]
-                    received = received[:i]
-                    break
-            received = decode_py2(received, 'utf-8')
+        if self.state != PROXY_CONNECTING or self.proxy_dict['type'] != \
+        'socks5':
+            # try to decode data
+            try:
+                received = decode_py2(received, 'utf-8')
+            except UnicodeDecodeError:
+                for i in range(-1, -4, -1):
+                    char = received[i]
+                    if sys.version_info[0] < 3: # with py2 we get a str
+                        char = ord(char)
+                    if char & 0xc0 == 0xc0:
+                        self.received_bytes_buff = received[i:]
+                        received = received[:i]
+                        break
+                received = decode_py2(received, 'utf-8')
 
         # pass received data to owner
         if self.on_receive:
@@ -702,7 +707,7 @@ class NonBlockingHTTP(NonBlockingTCP):
         :param http_dict: dictionary with data for HTTP request and headers
         """
         NonBlockingTCP.__init__(self, raise_event, on_disconnect, idlequeue,
-            estabilish_tls, certs, tls_version, cipher_list, proxy_dict)
+            estabilish_tls, certs, tls_version, cipher_list, False, proxy_dict)
 
         self.http_protocol, self.http_host, self.http_port, self.http_path = \
             urisplit(http_dict['http_uri'])
